local function MakeVegStats(seedweight, hunger, health, perish_time, sanity, cooked_hunger, cooked_health, cooked_perish_time, cooked_sanity, id, true_name)
	return 
	{
		health = health,
		hunger = hunger,
		cooked_health = cooked_health,
		cooked_hunger = cooked_hunger,
		seed_weight = seedweight,
		perishtime = perish_time,
		cooked_perishtime = cooked_perish_time,
		sanity = sanity,
		cooked_sanity = cooked_sanity,
		id = id,
		true_name = true_name,
	}
end

local COMMON = 2
local UNCOMMON = 1
local RARE = .5

EXTRA_VEGGIES = 
{
	tomato = MakeVegStats(COMMON, TUNING.CALORIES_SMALL, TUNING.HEALING_TINY,	TUNING.PERISH_FAST, 0, TUNING.CALORIES_MEDSMALL, TUNING.HEALING_SMALL, TUNING.PERISH_FAST, 0, 3, "tomato"),
	turnip = MakeVegStats(UNCOMMON, TUNING.CALORIES_SMALL, 0,	TUNING.PERISH_SLOW, 0, TUNING.CALORIES_MEDSMALL, 0, TUNING.PERISH_SLOW, 0, 5, "turnip"),
	potato = MakeVegStats(COMMON, TUNING.CALORIES_TINY/2, TUNING.HEALING_TINY,	TUNING.PERISH_SLOW, 0, TUNING.CALORIES_MEDSMALL, TUNING.HEALING_TINY, TUNING.PERISH_SLOW, TUNING.SANITY_SUPERTINY, 2, "potato"),
	garlic = MakeVegStats(UNCOMMON, TUNING.CALORIES_SMALL, 0,	TUNING.PERISH_SLOW, 0, TUNING.CALORIES_SMALL, TUNING.HEALING_MEDSMALL, TUNING.PERISH_SLOW, 0, 7, "garlic"),
	onion = MakeVegStats(UNCOMMON, TUNING.CALORIES_SMALL, 0,	TUNING.PERISH_MED, TUNING.SANITY_SUPERTINY, TUNING.CALORIES_MEDSMALL, TUNING.HEALING_TINY, TUNING.PERISH_MED, TUNING.SANITY_SUPERTINY, 4, "onion"),		
	wheat = MakeVegStats(COMMON, TUNING.CALORIES_SMALL, 0, TUNING.PERISH_SUPERSLOW, 0, TUNING.CALORIES_MEDSMALL, TUNING.HEALING_TINY, TUNING.PERISH_SUPERSLOW, 0, 6, "wheat"),
}


local function MakeVeggie(name, has_seeds)

	local assets =
	{	
		Asset("ANIM", "anim/"..name..".zip"),
		Asset("IMAGE", "images/inventoryimages/tomato.tex"),
		Asset("ATLAS", "images/inventoryimages/tomato.xml"),
		Asset("IMAGE", "images/inventoryimages/tomato_cooked.tex"),
		Asset("ATLAS", "images/inventoryimages/tomato_cooked.xml"),
		Asset("IMAGE", "images/inventoryimages/tomato_seeds.tex"),
		Asset("ATLAS", "images/inventoryimages/tomato_seeds.xml"),
		Asset("IMAGE", "images/inventoryimages/turnip.tex"),
		Asset("ATLAS", "images/inventoryimages/turnip.xml"),
		Asset("IMAGE", "images/inventoryimages/turnip_cooked.tex"),
		Asset("ATLAS", "images/inventoryimages/turnip_cooked.xml"),
		Asset("IMAGE", "images/inventoryimages/turnip_seeds.tex"),
		Asset("ATLAS", "images/inventoryimages/turnip_seeds.xml"),
		Asset("IMAGE", "images/inventoryimages/potato.tex"),
		Asset("ATLAS", "images/inventoryimages/potato.xml"),
		Asset("IMAGE", "images/inventoryimages/potato_cooked.tex"),
		Asset("ATLAS", "images/inventoryimages/potato_cooked.xml"),
		Asset("IMAGE", "images/inventoryimages/potato_seeds.tex"),
		Asset("ATLAS", "images/inventoryimages/potato_seeds.xml"),
		Asset("IMAGE", "images/inventoryimages/wheat.tex"),
		Asset("ATLAS", "images/inventoryimages/wheat.xml"),
		Asset("IMAGE", "images/inventoryimages/wheat_cooked.tex"),
		Asset("ATLAS", "images/inventoryimages/wheat_cooked.xml"),
		Asset("IMAGE", "images/inventoryimages/wheat_seeds.tex"),
		Asset("ATLAS", "images/inventoryimages/wheat_seeds.xml"),
		Asset("IMAGE", "images/inventoryimages/garlic.tex"),
		Asset("ATLAS", "images/inventoryimages/garlic.xml"),
		Asset("IMAGE", "images/inventoryimages/garlic_cooked.tex"),
		Asset("ATLAS", "images/inventoryimages/garlic_cooked.xml"),
		Asset("IMAGE", "images/inventoryimages/garlic_seeds.tex"),
		Asset("ATLAS", "images/inventoryimages/garlic_seeds.xml"),
		Asset("IMAGE", "images/inventoryimages/onion.tex"),
		Asset("ATLAS", "images/inventoryimages/onion.xml"),
		Asset("IMAGE", "images/inventoryimages/onion_cooked.tex"),
		Asset("ATLAS", "images/inventoryimages/onion_cooked.xml"),
		Asset("IMAGE", "images/inventoryimages/onion_seeds.tex"),
		Asset("ATLAS", "images/inventoryimages/onion_seeds.xml"),
	}

	local assets_cooked = {}

	local assets_seeds = {}
		
	local prefabs =
	{	
		name.."_cooked",
		"spoiled_food",
	}
	
	if has_seeds then
		table.insert(prefabs, name.."_seeds")
	end

	local function fn_seeds()
		local inst = CreateEntity()

		inst.entity:AddTransform()
		inst.entity:AddAnimState()
	    
		MakeInventoryPhysics(inst)

        inst.AnimState:SetBank("seeds")
        inst.AnimState:SetBuild("seeds")
        inst.AnimState:SetRayTestOnBB(true)
		inst.AnimState:PlayAnimation("idle")
		
		if IsDLCEnabled(CAPY_DLC) then
			MakeInventoryFloatable(inst, "idle_water", "idle")
		end

		inst:AddComponent("edible")
		inst.components.edible.healthvalue 	= TUNING.HEALING_TINY/2
		inst.components.edible.hungervalue 	= TUNING.CALORIES_TINY
		inst.components.edible.foodtype 	= "SEEDS"

        inst:AddComponent("stackable")
		inst.components.stackable.maxsize = TUNING.STACK_SIZE_SMALLITEM

		inst:AddComponent("tradable")
		inst:AddComponent("inspectable")
		inst:AddComponent("inventoryitem")
	    inst.components.inventoryitem.imagename = name.."_seeds"
	    inst.components.inventoryitem.atlasname = "images/inventoryimages/"..name.."_seeds.xml"

		inst:AddComponent("perishable")
		inst.components.perishable:SetPerishTime(TUNING.PERISH_SUPERSLOW)
		inst.components.perishable:StartPerishing()
		inst.components.perishable.onperishreplacement = "spoiled_food"

		inst:AddComponent("cookable")
		inst.components.cookable.product = "seeds_cooked"

		inst:AddComponent("bait")

		inst:AddComponent("plantable")
		inst.components.plantable.growtime = TUNING.SEEDS_GROW_TIME
		inst.components.plantable.product = name

		return inst
	end

	local function fn()
		local inst = CreateEntity()

		inst.entity:AddTransform()
		inst.entity:AddAnimState()

		MakeInventoryPhysics(inst)

		inst.AnimState:SetBank(name)
		inst.AnimState:SetBuild(name)
		inst.AnimState:PlayAnimation("idle")
		
		if IsDLCEnabled(CAPY_DLC) then
			MakeInventoryFloatable(inst,"idle","idle")
		end

		inst:AddComponent("edible")
		inst.components.edible.healthvalue 	= EXTRA_VEGGIES[name].health
		inst.components.edible.hungervalue 	= EXTRA_VEGGIES[name].hunger
		inst.components.edible.sanityvalue 	= EXTRA_VEGGIES[name].sanity or 0		
		inst.components.edible.foodtype 	= "VEGGIE"

        inst:AddComponent("stackable")
		inst.components.stackable.maxsize = TUNING.STACK_SIZE_SMALLITEM

		inst:AddComponent("tradable")
		inst:AddComponent("inspectable")
		inst:AddComponent("inventoryitem")
	    inst.components.inventoryitem.imagename = name
	    inst.components.inventoryitem.atlasname = "images/inventoryimages/"..name..".xml"

		inst:AddComponent("perishable")
		inst.components.perishable:SetPerishTime(EXTRA_VEGGIES[name].perishtime)
		inst.components.perishable:StartPerishing()
		inst.components.perishable.onperishreplacement = "spoiled_food"   

		inst:AddComponent("bait")

		inst:AddComponent("cookable")
		inst.components.cookable.product = name.."_cooked"

	    MakeSmallBurnable(inst)
		MakeSmallPropagator(inst) 

		return inst
	end
	
	local function fn_cooked()
		local inst = CreateEntity()

		inst.entity:AddTransform()
		inst.entity:AddAnimState()

		MakeInventoryPhysics(inst)
		
		inst.AnimState:SetBank(name)
		inst.AnimState:SetBuild(name)
		inst.AnimState:PlayAnimation("cooked")
		
		if IsDLCEnabled(CAPY_DLC) then
			MakeInventoryFloatable(inst,"cooked","cooked")
		end

		inst:AddComponent("edible")
		inst.components.edible.healthvalue = EXTRA_VEGGIES[name].cooked_health
		inst.components.edible.hungervalue = EXTRA_VEGGIES[name].cooked_hunger
		inst.components.edible.sanityvalue = EXTRA_VEGGIES[name].cooked_sanity or 0
		inst.components.edible.foodtype = "VEGGIE"

        inst:AddComponent("stackable")
		inst.components.stackable.maxsize = TUNING.STACK_SIZE_SMALLITEM

		inst:AddComponent("tradable")
		inst:AddComponent("inspectable")
		inst:AddComponent("inventoryitem")
	    inst.components.inventoryitem.imagename = name.."_cooked"
	    inst.components.inventoryitem.atlasname = "images/inventoryimages/"..name.."_cooked.xml"

		inst:AddComponent("perishable")
		inst.components.perishable:SetPerishTime(EXTRA_VEGGIES[name].cooked_perishtime)
		inst.components.perishable:StartPerishing()
		inst.components.perishable.onperishreplacement = "spoiled_food"

		inst:AddComponent("bait")

	    MakeSmallBurnable(inst)
		MakeSmallPropagator(inst)

		return inst
	end
	
	if TUNING.GORGE_CROPS == "gorge_crops_yes" then
	
		local base = Prefab( "common/inventory/"..name, fn, assets, prefabs)
		local cooked = Prefab( "common/inventory/"..name.."_cooked", fn_cooked, assets)
		local seeds = has_seeds and Prefab( "common/inventory/"..name.."_seeds", fn_seeds, assets) or nil
		
		return base, cooked, seeds
	end	
		
	return base, cooked, seeds
end

STRINGS.NAMES.TOMATO = "Tomato"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.TOMATO = "Tomato tomato."
STRINGS.NAMES.TOMATO_COOKED = "Cooked Tomato"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.TOMATO_COOKED = "Finely cooked tomato."
STRINGS.NAMES.TOMATO_SEEDS = "Tomato Seeds"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.TOMATO_SEEDS = "A bizzare set of seeds."
STRINGS.NAMES.ONION = "Onion"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.ONION = "A round and pungent root."
STRINGS.NAMES.ONION_COOKED = "Onion Rings"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.ONION_COOKED = "If you like it you should have put an onion ring on it."
STRINGS.NAMES.ONION_SEEDS = "Onion Seeds"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.ONION_SEEDS = "A bizzare set of seeds."
STRINGS.NAMES.POTATO = "Potato"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.POTATO = "A hard and starchy tuber."
STRINGS.NAMES.POTATO_COOKED = "Baked Potato"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.POTATO_COOKED = "A well cooked potato."
STRINGS.NAMES.POTATO_SEEDS = "Potato Seeds"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.POTATO_SEEDS = "A bizzare set of seeds."
STRINGS.NAMES.TURNIP = "Turnip"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.TURNIP = "A plain old root vegetable."
STRINGS.NAMES.TURNIP_COOKED = "Cooked Turnip"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.TURNIP_COOKED = "Still very plain."
STRINGS.NAMES.TURNIP_SEEDS = "Turnip Seeds"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.TURNIP_SEEDS = "A bizzare set of seeds."
STRINGS.NAMES.GARLIC = "Garlic"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.GARLIC = "A strong smelling yet good tasting root."
STRINGS.NAMES.GARLIC_COOKED = "Cooked Garlic"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.GARLIC_COOKED = "The odour is even stronger now."
STRINGS.NAMES.GARLIC_SEEDS = "Garlic Seeds"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.GARLIC_SEEDS = "A bizzare set of seeds."
STRINGS.NAMES.WHEAT = "Wheat"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.WHEAT = "Better grind this into flour."
STRINGS.NAMES.WHEAT_COOKED = "Cooked Wheat"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.WHEAT_COOKED = "Nicely toasted grains."
STRINGS.NAMES.WHEAT_SEEDS = "Wheat Seeds"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.WHEAT_SEEDS = "A bizzare set of seeds."

local prefs = {}
for veggiename,veggiedata in pairs(EXTRA_VEGGIES) do
	local veg, cooked, seeds

	veg, cooked, seeds = MakeVeggie(veggiename, true)

	table.insert(prefs, veg)
	table.insert(prefs, cooked)
	if seeds then
		table.insert(prefs, seeds)
		VEGGIES[veggiename] = veggiedata
	end
end

return unpack(prefs)