require "prefabutil"
require "tuning"

local assets =
{
	Asset("ANIM", "anim/farmplot.zip"),
	Asset("ANIM", "anim/quagmire_soil.zip"),
	Asset("ANIM", "anim/garden_square.zip"),
	Asset("ANIM", "anim/quagmire_spiceshrub.zip"),
	Asset( "IMAGE", "images/inventoryimages/veg_garden_useable.tex" ),
	Asset( "ATLAS", "images/inventoryimages/veg_garden_useable.xml" ),
}

local prefabs = 
{
	"plant_normal",
	"farmrock",
	"farmrocktall",
	"farmrockflat",
	"stick",
	"stickleft",
	"stickright",
	"signleft",
	"signright",
	"fencepost",
	"fencepostright",
}

local back = -1
local front = 0
local left = 1.5
local right = -1.5
    
local function onhammered(inst, worker)
    inst.components.grower:Reset()
	inst.components.lootdropper:DropLoot()
	SpawnPrefab("collapse_small").Transform:SetPosition(inst.Transform:GetWorldPosition())
	inst:Remove()
	inst.SoundEmitter:PlaySound("dontstarve/common/destroy_wood")
end

local function onhit(inst, worker)
	--inst.AnimState:PlayAnimation("hit")
	--inst.AnimState:PushAnimation("idle")
end

local function setfertilityfn(inst, fert_percent)
	
	local anim = "full"
	if fert_percent <= 0 then
		anim = "empty"
	elseif fert_percent <= .33 then
		anim = "med2"
	elseif fert_percent <= .66 then
		anim = "med1"
	end
	
	inst.AnimState:PlayAnimation(anim)
	
end

local rates = 
{
	TUNING.FARM1_GROW_BONUS,
	TUNING.FARM2_GROW_BONUS,
	TUNING.FARM3_GROW_BONUS,
}

local croppoints = 
{
	{ Vector3(0,0,0) },
	{ Vector3(0,0,0) },
	{ Vector3(0,0,0) },
}

function GetClosestInstWithTag(tag, inst, radius)
	local trans = inst.Transform
	local tags = {tag}
	local x,y,z = trans:GetWorldPosition()
	local ents = TheSim:FindEntities(x,y,z, radius, tags)
	for k,v in pairs(ents) do
		if v ~= inst then return v end
	end
end

local function PlotAnimation(inst, data)
	inst.AnimState:SetBank("quagmire_soil")
	inst.AnimState:SetBuild("quagmire_soil")
	inst.AnimState:PlayAnimation("risen")
	inst.AnimState:SetFinalOffset(2)
	inst.AnimState:Hide("mouseover")
	inst.AnimState:Hide("crop_bulb1")
	inst.AnimState:Hide("crop_bulb2")
	inst.AnimState:Hide("crop_bulb3")	
	
	--local x,y,z = inst.Transform:GetWorldPosition()
	--local ents = TheSim:FindEntities(x,y,z,3, {"veg_garden"}, {"FX", "NOCLICK", "DECOR","INLIMBO"})	
	--if not ents then
	--	inst:Remove()
	--end
	
	local veggarden = GetClosestInstWithTag("veg_garden",inst,60)
	if not veggarden then
		inst:Remove()
	end
end

local function plot(level)
	local rock_front = 1

    return function(Sim)
        local inst = CreateEntity()
        local trans = inst.entity:AddTransform()
        local anim = inst.entity:AddAnimState()
        inst.entity:AddSoundEmitter()
        
        --inst:AddTag("structure")
		inst:AddTag("farmplot_spot")
        
        --anim:SetBank("farmplot")
        --anim:SetBuild("farmplot")
        --anim:PlayAnimation( "full" )
		--anim:SetScale(0.33,0.33,0.33)
		--anim:SetOrientation( ANIM_ORIENTATION.OnGround )
		--anim:SetLayer( LAYER_BACKGROUND )
		--anim:SetSortOrder(3)		
		anim:SetBank("quagmire_soil")
		anim:SetBuild("quagmire_soil")
		anim:PlayAnimation("risen")
		anim:SetFinalOffset(2)
		anim:Hide("mouseover")
		anim:Hide("crop_bulb1")
		anim:Hide("crop_bulb2")
		anim:Hide("crop_bulb3")

	    local minimap = inst.entity:AddMiniMapEntity()
	    --minimap:SetIcon( "farm"..level..".png" )

        inst:AddComponent("inspectable")
        inst.components.inspectable.nameoverride = "FARMPLOT"
        inst.components.inspectable.getstatus = function(inst)
            if not inst.components.grower:IsFertile() then
				return "NEEDSFERTILIZER"
            elseif not inst.components.grower:IsEmpty() then
                return "GROWING"
            end
        end
        
        inst:AddComponent("grower")
        inst.components.grower.level = level
        inst.components.grower.onplantfn = function() inst.SoundEmitter:PlaySound("dontstarve/wilson/plant_seeds") end
        inst.components.grower.croppoints = croppoints[level]
        inst.components.grower.growrate = 0.66
        
        local cycles_per_level = {10,20,30}
		
        inst.components.grower.max_cycles_left = cycles_per_level[level] or 6
        inst.components.grower.cycles_left = inst.components.grower.max_cycles_left
		inst.components.grower.setfertility = setfertilityfn
		inst.decor = {}

		trans:SetRotation(45)
		
		inst:DoPeriodicTask(0.5, PlotAnimation)

		inst:AddComponent("lootdropper")      
        return inst
    end
end    

local function gardenrocks()
    local inst = CreateEntity()
    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()	
	inst.persists = false
    inst.AnimState:SetBank("farm_decor")
    inst.AnimState:SetBuild("farm_decor")
	local rnd = math.random()
	if rnd <= 0.2 then
		inst.AnimState:PlayAnimation("8")
	elseif rnd <= 0.6 then
		inst.AnimState:PlayAnimation("1")
	else
		inst.AnimState:PlayAnimation("2")
	end
    return inst
end

local function OnBuilt(inst)
    inst.SoundEmitter:PlaySound("dontstarve/common/farm_improved_craft")
end

local function OnSave(inst, data)
	if inst.isloaded then
		data.isloaded = inst.isloaded ~= nil
	end
end

local function OnLoad(inst, data)
	if data ~= nil then
		if data.isloaded then
			inst.isloaded = data.isloaded 
		end
	end
end

local function FillGarden(inst, num, k)
	local x, y, z = inst.Transform:GetWorldPosition()
	for i = -num, num do
		SpawnPrefab("farmplot_spot").Transform:SetPosition(x - i * inst.density, 0.32, z - k * inst.density)
	end
end

local function SetStones(inst, num, width, height)
	local x, y, z = inst.Transform:GetWorldPosition()
	local total_edge_length = width*2 + height*2
	local dist_per_item = total_edge_length/num
	local s = 0.741	
	for i = 1, num do
	   	local current_pos = i * dist_per_item		
	   	if current_pos < width then
			local rocks = SpawnPrefab("veg_garden_rocks")
			rocks.entity:SetParent(inst.entity)
			rocks.Transform:SetPosition(-width/2.0+current_pos, 0, -height/2.0)
			rocks.Transform:SetScale(s,s,s)
		else
			current_pos = current_pos - width
	   		if current_pos < height then
				local rocks = SpawnPrefab("veg_garden_rocks")		
				rocks.entity:SetParent(inst.entity)
				rocks.Transform:SetPosition(width/2.0, 0,-height/2.0+current_pos)
				rocks.Transform:SetScale(s,s,s)
			else
				current_pos = current_pos - height			
			   	if current_pos < width then
					local rocks = SpawnPrefab("veg_garden_rocks")			
					rocks.entity:SetParent(inst.entity)
					rocks.Transform:SetPosition(width/2.0-current_pos, 0, height/2.0)
					rocks.Transform:SetScale(s,s,s)
				else
					current_pos = current_pos - width
					local rocks = SpawnPrefab("veg_garden_rocks")					
					rocks.entity:SetParent(inst.entity)
					rocks.Transform:SetPosition(-width/2.0, 0, height/2.0-current_pos)
					rocks.Transform:SetScale(s,s,s)
				end
			end
		end
	end
	if inst.isloaded == nil then
		for k = -inst.amountofcropy, inst.amountofcropy do
			FillGarden(inst, inst.amountofcropx, k)
		end
		inst.isloaded = true
	end
end

local function onhammered(inst, worker)
    if inst.components.burnable ~= nil and inst.components.burnable:IsBurning() then
        inst.components.burnable:Extinguish()
    end
	
	local pos = inst:GetPosition()
	local ents = TheSim:FindEntities(pos.x, pos.y, pos.z, 3, nil, {"FX","DECOR","INLIMBO"}, {"farmplot_spot"})
	for i,v in pairs(ents) do
		if v then
			v:Remove()
		end
	end
	
	if inst.components.lootdropper ~= nil then
        inst.components.lootdropper:DropLoot()
    end
	
    local fx = SpawnPrefab("collapse_small")
    fx.Transform:SetPosition(inst.Transform:GetWorldPosition())
    inst:Remove()
end

local function garden_fn(bank)
    local inst = CreateEntity()
    inst.entity:AddTransform()
    inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()

	inst.Transform:SetTwoFaced()
	
	inst:AddTag("veg_garden")
	
	local minimap = inst.entity:AddMiniMapEntity()
	minimap:SetIcon("veg_garden_useable.tex")
	
    inst.AnimState:SetBank("garden_square")
    inst.AnimState:SetBuild("garden_square")
    inst.AnimState:PlayAnimation("idle")
    inst.AnimState:SetLayer(LAYER_BACKGROUND)
	inst.AnimState:SetOrientation(ANIM_ORIENTATION.OnGround)
    inst.AnimState:SetSortOrder(2)

	local s = 1.35
	inst.Transform:SetScale(s,s,s)
		
	inst:AddComponent("lootdropper")

	inst:AddComponent("workable")
	inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
	inst.components.workable:SetWorkLeft(8)
	inst.components.workable:SetOnFinishCallback(onhammered)
	
	inst.num = 44
	inst.width = 4.5
	inst.height = 4.5
	inst.amountofcropx = 1
	inst.amountofcropy = 1
	inst.density = 2
	
	inst:DoTaskInTime(0, SetStones, inst.num, inst.width, inst.height)
	
	inst:ListenForEvent("onbuilt", OnBuilt)
	
	inst.isloaded = nil
	
	inst.OnSave = OnSave
	inst.OnLoad = OnLoad

    return inst
end

STRINGS.NAMES.VEG_GARDEN_USEABLE = "Vegetable Garden"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.VEG_GARDEN_USEABLE = "A nice big garden."
STRINGS.RECIPE_DESC.VEG_GARDEN_USEABLE = "Large garden with multiple farm spots."

STRINGS.NAMES.FARMPLOT_SPOT = "Arable Soil"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.FARMPLOT_SPOT = "The ground is quite fertile here."

return Prefab( "common/objects/farmplot_spot",  plot(2), assets, prefabs ),
       Prefab("veg_garden_rocks", gardenrocks, assets),
	   Prefab("veg_garden_useable", garden_fn, assets),
	   MakePlacer("veg_garden_useable_placer", "garden_square", "garden_square", "idle", true, nil, nil, 1.35, nil, nil, nil)